#!/usr/bin/env python
# -*- coding: utf-8 -*-
from pathlib import Path
from setuptools import find_packages, setup

# Package meta-data.
NAME = 'attendance-model'
DESCRIPTION = "An end to end machine learning app to predict canteens attendance 2-3 weeks ahead in Nantes Metropole"
URL = "https://github.com/qangelot/projet_Nantes"
EMAIL = "q.angelot@gmail.com"
AUTHOR = "qangelot"
REQUIRES_PYTHON = ">=3.6.0"


# ------------------------------------------------
# If changing the License, remember to change the
# Trove Classifier for that!
long_description = DESCRIPTION

# package's VERSION file in a dictionary
about = {}
ROOT_DIR = Path(__file__).resolve().parent
REQUIREMENTS_DIR = ROOT_DIR / 'requirements'
PACKAGE_DIR = ROOT_DIR / 'attendance_model'
with open(PACKAGE_DIR / "VERSION") as f:
   _version = f.read().strip()
   about["__version__"] = _version


# libraries required for module execution
def list_reqs(fname="requirements.txt"):
   with open(REQUIREMENTS_DIR / fname) as fd:
       return fd.read().splitlines()

# Where the magic happens:
setup(
   name=NAME,
   version=about["__version__"],
   description=DESCRIPTION,
   long_description=long_description,
   long_description_content_type="text/markdown",
   author=AUTHOR,
   author_email=EMAIL,
   python_requires=REQUIRES_PYTHON,
   url=URL,
   packages=find_packages(exclude=("tests",)),
   package_data={"attendance_model": ["VERSION"]},
   install_requires=list_reqs(),
   extras_require={},
   include_package_data=True,
   license="BSD-3",
   classifiers=[
       # Trove classifiers
       # standardized classifiers used by community members
       # to find projects based on their desired criteria
       "License :: OSI Approved :: MIT License",
       "Programming Language :: Python",
       "Programming Language :: Python :: 3",
       "Programming Language :: Python :: 3.6",
       "Programming Language :: Python :: 3.7",
       "Programming Language :: Python :: 3.8",
       "Programming Language :: Python :: 3.9",
       "Programming Language :: Python :: Implementation :: CPython",
       "Programming Language :: Python :: Implementation :: PyPy",
   ],
)